import 'package:copy_with_extension/copy_with_extension.dart';
import 'package:flutter/material.dart';
import 'package:healthrian_visualization_support/healthrian_visualization_support.dart';

part 'ble_scanner.g.dart';

_$BleScannerCWProxy get ScannerWearEcg =>
    BleScanner(module: BLEManager().wearEcg12.module).copyWith;

_$BleScannerCWProxy get ScannerCardioClick =>
    BleScanner(module: BLEManager().cardioClick.module).copyWith;

_$BleScannerCWProxy get ScannerWearT =>
    BleScanner(module: BLEManager().wearCardioT.module).copyWith;

@CopyWith()
class BleScanner extends StatefulWidget {
  const BleScanner({
    super.key,
    this.module,
    this.connectedBuilder,
    this.scannedBuilder,
    this.onTapConnected,
    this.onTapScanned,
  });

  final ModuleInterface? module;

  final Function(BluetoothDevice)? onTapConnected;
  final Function(ScanResult)? onTapScanned;
  final Widget Function(BluetoothDevice)? connectedBuilder;
  final Widget Function(ScanResult)? scannedBuilder;

  @override
  State<BleScanner> createState() => _BleScannerState();
}

class _BleScannerState extends State<BleScanner> {
  @override
  Widget build(BuildContext context) {
    final module = widget.module ?? BLEManager().scanner;
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: StreamBuilder(stream: () async* {
        final stream = Stream.periodic(Duration(seconds: 4));
        await for (final _ in stream) {
          final connected = await module.ble.findConnectedDevices();
          final scanned = await module.ble.findScannedDevices();

          yield (connected, scanned);
        }
      }(), builder: (context, snapshot) {
        final connected = snapshot.data?.$1 ?? [];
        final scanned = snapshot.data?.$2 ?? [];
        return ListView.builder(
          itemCount: scanned.length + connected.length,
          itemBuilder: (context, index) {
            if (index < connected.length) {
              final device = connected[index];
              return InkWell(
                onTap: () async {
                  await widget.onTapConnected?.call(device) ??
                      module.ble.disconnectDevice(device: device);
                  setState(() {});
                },
                child: widget.connectedBuilder?.call(device) ??
                    Card(
                      child: SizedBox(
                        height: 80,
                        child: ListTile(
                          title: Text('${device.platformName}'),
                          titleTextStyle: TextStyle(fontSize: 24),
                          subtitle: Text(device.remoteId.str),
                          subtitleTextStyle: TextStyle(fontSize: 16),
                          trailing: Text('Connected'),
                        ),
                      ),
                    ),
              );
            }
            final scannedIndex = index - connected.length;
            final scanResult = scanned[scannedIndex];

            return InkWell(
              onTap: () async {
                await widget.onTapScanned?.call(scanResult) ??
                    module.ble
                        .connectDevice(scanResult.device.remoteId.str, true);
                setState(() {});
              },
              child: widget.scannedBuilder?.call(scanResult) ??
                  Card(
                    child: SizedBox(
                      height: 80,
                      child: ListTile(
                        title: Text('${scanResult.device.platformName}'),
                        titleTextStyle: TextStyle(fontSize: 24),
                        subtitle: Text(scanResult.device.remoteId.str),
                        subtitleTextStyle: TextStyle(fontSize: 16),
                        trailing: Text('Scanned'),
                      ),
                    ),
                  ),
            );
          },
        );
      }),
    );
  }
}
